@@ -1,8 +1,8 @@
 /*
  * Linux block devices helpers
  *
- * Copyright (C) 2018-2019 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2018-2019 Ondrej Kozina
+ * Copyright (C) 2018-2021 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2018-2021 Ondrej Kozina
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -103,11 +103,11 @@ static int lookup_holder_dm_name(const char *dm_uuid, size_t max_len, dev_t devn
 		}
 
 		/* looking for dm-X/dm directory, symlinks are fine */
-		dmfd = openat(dirfd(dir), dm_subpath, O_DIRECTORY | O_RDONLY | O_CLOEXEC);
+		dmfd = openat(dirfd(dir), dm_subpath, O_DIRECTORY | O_RDONLY);
 		if (dmfd < 0)
 			continue;
 
-		fd = openat(dmfd, "uuid", O_RDONLY | O_CLOEXEC);
+		fd = openat(dmfd, "uuid", O_RDONLY);
 		if (fd < 0) {
 			close(dmfd);
 			continue;
@@ -130,7 +130,7 @@ static int lookup_holder_dm_name(const char *dm_uuid, size_t max_len, dev_t devn
 			continue;
 		}
 
-		fd = openat(dmfd, "name", O_RDONLY | O_CLOEXEC);
+		fd = openat(dmfd, "name", O_RDONLY);
 		if (fd < 0) {
 			close(dmfd);
 			continue;
