@@ -1,9 +1,9 @@
 /*
  * cryptsetup library API check functions
  *
- * Copyright (C) 2009-2018 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2018, Milan Broz
- * Copyright (C) 2016-2018, Ondrej Kozina
+ * Copyright (C) 2009-2021 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2021 Milan Broz
+ * Copyright (C) 2016-2021 Ondrej Kozina
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,6 +36,8 @@ extern uint64_t t_dev_offset;
 int t_device_size(const char *device, uint64_t *size);
 int t_dm_check_versions(void);
 int t_dm_crypt_keyring_support(void);
+int t_dm_crypt_cpu_switch_support(void);
+int t_dm_crypt_discard_support(void);
 
 int fips_mode(void);
 
@@ -56,6 +58,7 @@ int _system(const char *command, int warn);
 void register_cleanup(void (*cleanup)(void));
 
 void check_ok(int status, int line, const char *func);
+void check_ok_return(int status, int line, const char *func);
 void check_ko(int status, int line, const char *func);
 void check_equal(int line, const char *func, int64_t x, int64_t y);
 void check_null(int line, const char *func, const void *x);
@@ -66,6 +69,9 @@ void xlog(const char *msg, const char *tst, const char *func, int line, const ch
 #define OK_(x)		do { xlog("(success)", #x, __FUNCTION__, __LINE__, NULL); \
 			     check_ok((x), __LINE__, __FUNCTION__); \
 			} while(0)
+#define NOTFAIL_(x, y)	do { xlog("(notfail)", #x, __FUNCTION__, __LINE__, y); \
+			     check_ok_return((x), __LINE__, __FUNCTION__); \
+			} while(0)
 #define FAIL_(x, y)	do { xlog("(fail)   ", #x, __FUNCTION__, __LINE__, y); \
 			     check_ko((x), __LINE__, __FUNCTION__); \
 			} while(0)
@@ -83,9 +89,11 @@ void xlog(const char *msg, const char *tst, const char *func, int line, const ch
 			     printf("%s: %s\n", #x, (y)); x(); \
 			} while (0)
 
+#define CRYPT_FREE(x) do { crypt_free(x); x = NULL; } while (0)
+
 #define SECTOR_SHIFT 9L
 #define SECTOR_SIZE 512
-#define TST_LOOP_FILE_SIZE (((1<<20)*50)>>SECTOR_SHIFT)
+#define TST_LOOP_FILE_SIZE (((1<<20)*100)>>SECTOR_SHIFT)
 #define DIV_ROUND_UP(n,d) (((n) + (d) - 1) / (d))
 #define DIV_ROUND_UP_MODULO(n,d) (DIV_ROUND_UP(n,d)*(d))
 
